package org.distributeme.test.jsonrpc.jsonrpc.generated;

import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;
import org.distributeme.core.RegistryUtil;
import org.distributeme.core.RegistryLocation;
import org.distributeme.core.ServiceDescriptor;
import org.distributeme.core.ServiceDescriptor.Protocol;
import net.anotheria.anoprise.metafactory.MetaFactory;
import net.anotheria.anoprise.metafactory.FactoryNotFoundException;
import net.anotheria.anoprise.metafactory.Extension;
import net.anotheria.util.PidTools;
import net.anotheria.util.IdCodeGenerator;
import org.distributeme.core.RMIRegistryUtil;
import net.anotheria.anoprise.metafactory.ServiceFactory;
import org.distributeme.core.lifecycle.LifecycleComponentImpl;
import org.distributeme.core.Verbosity;
import org.distributeme.core.SystemPropertyNames;
import org.distributeme.core.ServerShutdownHook;
import java.security.Permission;
import cz.eman.jsonrpc.server.http.JsonRpcServlet;
import org.mortbay.jetty.security.SslSocketConnector;
import java.lang.System;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.servlet.Context;
import org.mortbay.jetty.servlet.ServletHolder;
import org.mortbay.jetty.nio.SelectChannelConnector;

import org.distributeme.support.lifecycle.generated.LifecycleSupportServer;
import org.distributeme.support.eventservice.generated.EventServiceRMIBridgeServer;
public class EchoServer{

	private static Logger log;

	public static void main(String a[]) throws Exception{
		DOMConfigurator.configureAndWatch("/log4j.xml");
		if (System.getSecurityManager()==null)
			System.setSecurityManager(new SecurityManager(){
				public void checkPermission(Permission perm) { }
			});
		init();
		// force verbosity to configure itself
		Verbosity.logServerSideExceptions();
		createSupportServicesAndRegisterLocally();
		createServiceAndRegisterLocally();
		startService();
	} //...main

	public static void init() throws Exception{
		log = Logger.getLogger(EchoServer.class);
		// // CUSTOM CODE STARTED
		// // CUSTOM CODE ENDED
	} //...init

	// Have to keep local reference to the clientStub to prevent gc removal
	private static ClientEchoService clientStub = null;
	private static String serviceId = null;

	public static void createServiceAndRegisterLocally() throws Exception{
		// creating impl
		// No factory specified
		try{
			Class<ServiceFactory<org.distributeme.test.jsonrpc.EchoService>> factoryClazz = (Class<ServiceFactory<org.distributeme.test.jsonrpc.EchoService>>)Class.forName("org.distributeme.test.jsonrpc.EchoServiceFactory");
			MetaFactory.addFactoryClass(org.distributeme.test.jsonrpc.EchoService.class, Extension.LOCAL, factoryClazz);
		}catch(ClassNotFoundException factoryNotFound){
			try{
				// Even more convinient - try to instantiate the implementation directly
				Class<? extends org.distributeme.test.jsonrpc.EchoService> implClazz = (Class<? extends org.distributeme.test.jsonrpc.EchoService>)Class.forName("org.distributeme.test.jsonrpc.EchoServiceImpl");
				MetaFactory.createOnTheFlyFactory(org.distributeme.test.jsonrpc.EchoService.class, Extension.LOCAL, implClazz.newInstance());
			}catch(ClassNotFoundException implNotFound){
				log.info("Giving up trying to find an impl instance, tried org.distributeme.test.jsonrpc.EchoServiceFactory and org.distributeme.test.jsonrpc.EchoServiceImpl, expect start to fail since init code were empty too and no factory has been supplied explicitely");
			} //...inner catch
		} //...outer catch
		org.distributeme.test.jsonrpc.EchoService impl = null;
		try{
			impl = MetaFactory.get(org.distributeme.test.jsonrpc.EchoService.class, Extension.LOCAL);
		}catch (FactoryNotFoundException factoryNotFound){
			throw new AssertionError("Un- or mis-configured, can't instantiate service instance for org.distributeme.test.jsonrpc.EchoService tried initcode, submitted factory, autoguessed factory (org.distributeme.test.jsonrpc.EchoServiceFactory) and impl class (org.distributeme.test.jsonrpc.EchoServiceImpl)");
		}
		clientStub = new ServerEchoServiceImpl(impl);
		serviceId = EchoServiceConstants.getServiceId();

		String regNameProviderClass = System.getProperty(SystemPropertyNames.REGISTRATION_NAME_PROVIDER);
		if (regNameProviderClass!=null){
			org.distributeme.core.routing.RegistrationNameProvider suppliedNameProvider = (org.distributeme.core.routing.RegistrationNameProvider)Class.forName(regNameProviderClass).newInstance();
			serviceId = suppliedNameProvider.getRegistrationName(serviceId);
		} //...if (regNameProviderClass!=null)

		LifecycleComponentImpl.INSTANCE.registerPublicService(serviceId, clientStub);
	}

	public static ServiceDescriptor createDescriptor(String instanceId) throws Exception{
		return RegistryUtil.createLocalServiceDescription(Protocol.JSONRPC,  serviceId, instanceId, RMIRegistryUtil.getRmiRegistryPort() +100);
	}
	public static void startService() throws Exception{
		// Log current server PID (Process Id)
		PidTools.logPid();
		String instanceId = IdCodeGenerator.generateCode(10);
		ServiceDescriptor descriptor = createDescriptor(instanceId);
		startHttpEndpoint(descriptor);

		if (!RegistryUtil.bind(descriptor)){
			log.fatal("Coulnd't bind myself to the central registry at "+RegistryUtil.describeRegistry());
			System.err.println("Coulnd't bind myself at the central registry at "+RegistryUtil.describeRegistry());
			System.exit(-3);
		} //...central registry bind
		Runtime.getRuntime().addShutdownHook(new ServerShutdownHook(descriptor));

		System.out.println("Server "+serviceId+" is up and ready.");
	} //...startService
	private static void startHttpEndpoint(final ServiceDescriptor descriptor) throws Exception {
		final Server server = new Server();
		if(System.getProperty("distributeme.json.ssl") != null) {
			final SslSocketConnector connector = new SslSocketConnector();
			if(System.getProperty("distributeme.json.ssl.password") != null) {
				connector.setPassword(System.getProperty("distributeme.json.ssl.password"));
			}
			if(System.getProperty("distributeme.json.ssl.keystore") != null) {
				connector.setKeystore(System.getProperty("distributeme.json.ssl.keystore"));
			}
			if(System.getProperty("distributeme.json.ssl.keyPassword") != null) {
				connector.setKeyPassword(System.getProperty("distributeme.json.ssl.keyPassword"));
			}
			if(System.getProperty("distributeme.json.ssl.truststore") != null) {
				connector.setTruststore(System.getProperty("distributeme.json.ssl.truststore"));
			}
			if(System.getProperty("distributeme.json.ssl.trustPassword") != null) {
				connector.setTrustPassword(System.getProperty("distributeme.json.ssl.trustPassword"));
			}
			connector.setPort(descriptor.getPort());
			server.addConnector(connector);
		} else {
			final SelectChannelConnector connector = new SelectChannelConnector();
			connector.setPort(descriptor.getPort());
			server.addConnector(connector);
		}
		final Context context = new Context(server, "/servlets", Context.SESSIONS);
		ServletHolder servletHolder = new ServletHolder(new JsonRpcServlet());
		servletHolder.setInitParameter(descriptor.getInstanceId(), clientStub.getClass().getName());
		context.addServlet(servletHolder, "/" + descriptor.getInstanceId());
		server.start();
	} //...startHttpEndpoint

	public static void createSupportServicesAndRegisterLocally() throws Exception{
		org.distributeme.support.lifecycle.generated.LifecycleSupportServer.init();
		org.distributeme.support.lifecycle.generated.LifecycleSupportServer.createServiceAndRegisterLocally();
		org.distributeme.support.eventservice.generated.EventServiceRMIBridgeServer.init();
		org.distributeme.support.eventservice.generated.EventServiceRMIBridgeServer.createServiceAndRegisterLocally();
	} //...createSupportServicesAndRegisterLocally

}
